import * as React from 'react';
import ReactTable, {Column, Filter, ReactTableFunction} from "react-table";
import {t} from "@lingui/macro";
import {JobDataSides, Side} from "../../pages/job_history/JobHistoryOperatorForm";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {SelectField} from "@rx-suite/ui";

type JobDataTableProps = {
    jobDataSides: JobDataSides;
};

export const JobDataTable: React.FunctionComponent<JobDataTableProps> = ({ jobDataSides }) => {

    // display the job data inside a table
    // NOTE: There could be collisions between the tag names due to the LMS data.
    // For this reason, using a hash may overwrite some values.
    // This may need to be reworked if this is the case.
    // group the left and right jobs by tag name

    const displayObj: {
        [tagName: string]: {
            [sideKey in Side]: string;
        } & {
            type: 'tag'|'process'|'base';
        }
    } = {};

    // iterate over each side of the job

    for (const side of [Side.LEFT, Side.RIGHT]) {
        const jobDataArray = jobDataSides[side];
        if (!jobDataArray) continue; // no information for this job side
        // iterate over each tag name/value of this job side
        for (const jobData of jobDataArray) {
            const tagName = jobData.name;
            if (!tagName || !tagName.length) continue; // no tag name associated to value
            const value = jobData.value;
            if (!(tagName in displayObj)) {
                // entry for tag name does not exist, create it
                displayObj[tagName] = {
                    left: '',
                    right: '',
                    type: jobData.type
                };
            }
            if (!value) continue; // if the value is null, keep the default string
            // set the value associated to this tag name for this job side
            displayObj[tagName][side] = value.toString();
        }
    }

    // create an array of data for the react table
    const tableData = Object.keys(displayObj).map(tagKey => {
        const displayObjValues = displayObj[tagKey];
        return {
            tag: tagKey,
            type: displayObjValues.type,
            left: displayObjValues.left,
            right: displayObjValues.right
        };
    });

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    const leftColumn = {
        Header: t`Left`,
        accessor: 'left'
    };

    const rightColumn = {
        Header: t`Right`,
        accessor: 'right'
    };

    const columns = [{
        Header: t`Tag`,
        accessor: 'tag',
        filterable: true
    },
    {
        Header: t`Type`,
        accessor: 'type',
        filterable: true,
        Filter: ({ filter, onChange }: {filter: Filter, onChange: ReactTableFunction}) => (
            <SelectField
                options={
                    [
                        {label: t`Base`, value: 'base'},
                        {label: t`Tag`, value: 'tag'},
                        {label: t`Process`, value: 'process'},
                        {label: t`All`, value: ""}
                    ]
                }
                onChange={(name, value) => onChange(value)}
                value={filter ? filter.value : ""}
            />
        )
    },
    displayRightLensFirst ? rightColumn : leftColumn,
    displayRightLensFirst ? leftColumn : rightColumn
    ] as Column[];

    return <ReactTable
        data={tableData}
        columns={columns}
        showPagination={false}
        // provide all of the data to the table,
        // without this, the table could show 30 rows,
        // but only fill 20 rows with data
        defaultPageSize={tableData.length}
        minRows={tableData.length <= 0 ? 10 : tableData.length} // if empty, pad the table with 10 rows
        sortable={false}
    />;
};
