import React, {useCallback, useEffect} from "react";
import {
    FormProps,
    makeColumn,
    ModelForm,
    ModelInput,
    ModelTableDefaultActions
} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import ReactTable from "react-table";
import {Typography, Button} from "@material-ui/core";
import {JobModel, JobStore} from "../../stores/JobStore";
import {Model} from "@rx-suite/data";
import {makeStyles} from "@material-ui/core/styles";
import {DataArchiveSettingStore} from "../../stores/DataArchiveSettingStore";
import {useSnackbar} from "notistack";
import {OrderModel} from "../../stores/OrderStore";

const useStyles = makeStyles(() => ({
    expiredOrder: {
        display: 'flex',
        flexDirection: 'row',
        justifyContent: 'flex-start',
        alignItems: 'center',
        margin: '5px 0px',
    },
    reactivateButton: {
        padding: '2px 8px',
        marginLeft: '8px'
    }
}));

export const OrderForm: React.FunctionComponent<FormProps> = observer((props) => {

    const model = props.model as OrderModel;
    const readOnly = props.readOnly;

    const classes = useStyles();

    const settings = DataArchiveSettingStore.getAll()[0];

    const {enqueueSnackbar} = useSnackbar();

    const [jobModels, setJobModels] = React.useState<(JobModel | undefined)[]>([]);

    const [reviving, setReviving] = React.useState<boolean>(false);

    useEffect(() => {
        const resp = JobStore.getByIds(model.job_ids.map(j => j.toString()));
        resp.then((data) => {
            setJobModels(data);
        });
    }, [model.job_ids]);

    const handleRevive = useCallback(async () => {
        if(!settings) {
            return enqueueSnackbar(`${t`Unable to reactivate the order number.`}`, {variant: "error"})
        }
        try {
            setReviving(true);
            await fetch(`/api/orders/${model.id}/revive`, {
                method: "PATCH",
                headers: {'Content-Type': 'application/json'}
            }).then(resp => resp.json()).then(data => {
                if (data.error) throw data.error;

                enqueueSnackbar(t`Order Revived`, {variant: "success"});
            });

        } catch (err) {
            const message = (err === 'jobAlreadyActive'
                ? t`Job number already in use`
                : t`Could not revive Order`
            );
            enqueueSnackbar(message, {variant: "error"});
        } finally {
            setReviving(false);
        }
    }, [model, settings]);

    const jobColumns = React.useMemo(() => {
        return [{Header: t`Jobs`, columns: [
                makeColumn('number', t`Number`, "string"),
                makeColumn('name', t`Name`, "string"),
                makeColumn('description', t`Description`, "string"),
                makeColumn('sequence_number', t`Sequence Number`, "string"),
                makeColumn('priority', t`Priority`, "string"),
                makeColumn('manufacturer', t`Manufacturer`, "string"),
                makeColumn('side', t`Side`, "string"),
                makeColumn('lmat_id', t`LMAT ID`, "string"),
                makeColumn('lmat_type', t`LMAT TYPE`, "string"),
                makeColumn('batch_criteria', t`Batch Criteria`, "string"),
                makeColumn('template', t`Template`, "string"),
                makeColumn('job_type', t`Job Type`, "string"),
                makeColumn('enabled', t`Enabled`, "string"),
                {accessor: d => d, id: '_actions', style: {textAlign: "right"}, width: 200, Cell: (cellInfo: { value: Model }) => {
                    if (!cellInfo.value || !cellInfo.value.getId) {
                        return null;
                    }
                    return <>
                        <ModelTableDefaultActions store={JobStore} model={cellInfo.value} />
                    </>}}
            ]
        }]
    }, [jobModels])

    const isExpired = () => {
        const now = new Date();

        const expiry = new Date(model.expiry_date);

        return expiry < now;
    };

    return (
        <ModelForm model={model} readOnly={readOnly}>
            <ModelInput name={'job_number'}/>
            {isExpired() && <div className={classes.expiredOrder}>
                <Typography variant={"body1"}>{t`This order has expired`}</Typography>
                <Button
                    className={classes.reactivateButton}
                    size="large"
                    variant={"contained"}
                    color="secondary"
                    onClick={handleRevive}
                    disableElevation
                    disabled={reviving}
                >
                    {t`Reactivate`}
                </Button>
            </div>}
            <ReactTable
                data={jobModels}
                columns={jobColumns}
                defaultPageSize={5}
            />
        </ModelForm>
    )
})