module OrderCreateHelper
  extend ActiveSupport::Concern

  include NormalizeJobNumber

  class OrderWithNoAssociatedJobs < StandardError; end

  def create_order_and_jobs(order_id, add_params, address_params, ship_to_params, data_params, process_params)
    order = create_order(order_id, add_params, address_params, ship_to_params, [])
    jobs = []

    %w[l r].each_with_index do |side_letter, side_num|
      sided_add_params = {
        name: add_params[:name],
        side: side_num,
        description: add_params[:description],
        priority: add_params[:priority]
      }

      # Job data is always either do 'L' or do 'R', never do 'B'
      sided_data_params = data_params.select do |data|
        data[:side] == side_num
      end

      sided_process_params = []
      process_params.each do |process|
        if process[:eye] == 'B' || process[:eye].downcase == side_letter
          sided_process_params << process.except(:eye)
        end
      end

      # Only create the job if it has process data (if the job has job data but no process data, ignore it)
      if sided_process_params.length > 0
        jobs << create_job(order_id, sided_add_params, sided_data_params, sided_process_params, order_id)
      end
    end

    jobs = jobs.compact

    raise OrderWithNoAssociatedJobs if jobs.empty?

    # Return one job id to allow routing to hook into the order
    # Return order_id for orders created by Rx-Paperless
    {job_id: jobs[0].id, order_id: order.id}
  end

  def create_job(job_number, add_params, data_params, process_params, order_number = nil)
    Job.ensure_job_uniqueness(job_number, add_params[:side])
    unless order_number.nil?
      # an old order with the job number might have been expired by Job.ensure_job_uniqueness make sure to get/create an active order
      order = FindHelpers::OrderFinders.find_active_or_create_by_number(order_number)
    end
    job = Job.create(number: normalize_job_number(job_number), side: add_params[:side])
    job.update add_params
    job.job_data.build data_params
    job.manufacturing_processes.build process_params
    job.order = order
    job.broadcast if job.save
    job
  end

  def create_order(order_id, add_params, address_params, ship_to_params, data_params)
    Order.ensure_order_uniqueness(order_id)
    order = Order.create(job_number: order_id)
    order.update add_params

    address = Address.find_or_create_by(address_params)
    ship_to = Address.find_or_create_by(ship_to_params)

    order.address = address
    order.ship_to = ship_to

    order.order_data.build data_params
    order.broadcast if order.save
    order
  end
end