class ProcessArchiveTasksJob < ApplicationJob
  queue_as :archive

  def delete(id)
    ActiveRecord::Base.connected_to(shard: :primary, role: :writing) { ArchiveTask.delete(id) }
  end

  def perform
    # limit the amount of archive task done in one run, otherwise we may load too much into memory.
    tasks = ArchiveTask.limit(archive_task_limit).all.to_a

    ActiveRecord::Base.connected_to(shard: :archive, role: :writing) do
      tasks.each do |task|
        begin
          if task.creator?
            insert_all = ActiveRecord::InsertAll.new(task.target_class, [task.target_attributes], on_duplicate: :raise)
            ActiveRecord::Base.connection.execute(insert_all.send(:to_sql))
          else
            attributes = task.target_attributes
            id = attributes.delete "id"
            task.target_class.find(id).update_columns(attributes)
          end
          ActiveRecord::Base.connected_to(shard: :primary, role: :writing) { ArchiveTask.delete(task.id) }
        rescue => err
          is_allowed_retry = allow_retry(task, err)
          Rails.logger.error "#{err} - attempt #{task.retry_count}/#{max_retry} - is_allowed_retry: #{is_allowed_retry}"
          ActiveRecord::Base.connected_to(shard: :primary, role: :writing) {
            if is_allowed_retry
              task.increment(:retry_count, 1).save
            else
              ArchiveTask.delete(task.id)
            end
          }
        end
      end
    end
  end

  def allow_retry(task, err)
    # Don't bother retrying certain error classes
    do_not_retry = [ActiveRecord::RecordNotUnique, ActiveRecord::UnknownAttributeError, ActiveRecord::StatementInvalid]
    return false if do_not_retry.detect { |e| err.instance_of?(e) }
    return false if task.retry_count >= max_retry
    true
  end

  def max_retry
    5
  end

  private

  def archive_task_limit
    return ENV["ARCHIVE_TASK_LIMIT"].to_i unless ENV["ARCHIVE_TASK_LIMIT"].nil?
    50000
  end
end
