module MachineKpiCalculator
  class TopAborts < MachineKpiCalculator
    include KpiTableHandler
    include ErrorAndAbortPercentage

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options)
        kpi = TopAborts.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :abort_count,
          handler: KpiTableHandler::MachineAbortCountKpiHandler,
          params: {
          }
        }
      ]
    end

    def top_aborts?
      !!options[:top_count]
    end

    def top_count
      options[:top_count]
    end

    def as_percent?
      options[:as_percent]
    end

    def extract_top_x(abort_codes, top=5)
      top_aborts = []
      abort_codes.sort_by { |k, v| [-v, k] }.each do |abort_code, value|
        break if top_aborts.size >= top

        next if abort_code == 'none'
        next if ignore_abort_code?(abort_code)

        top_aborts << {code: abort_code, count: value}
      end
      top_aborts
    end

    def ignore_abort_code?(abort_code)
      # return false unless no_abort_filter?(abort_code)

      filter = AbortFilter.find_by_group_code(abort_code)
      return false if filter.nil?

      filter.is_ignore_filter?
    end

    def no_abort_filter?(abort_code)
      abort_code == "other"
    end

    def resolve_abort_code(abort_code)
      # If abort filter exists and replace behaviour
      unless no_abort_filter? abort_code
        filter = AbortFilter.find_by_group_code(abort_code)

        unless filter.nil?
          return filter.replace_name if filter.behaviour == 'replace'
        end
      end

      # No filter, filter is invalid, or filter is 'use-as-is'
      "Other"
    end

    # group all abort codes
    def group_codes(aborts_hash)
      abort_codes = {}
      aborts_hash.each do |time, data|
        abort_codes.merge!(data) { |k, a, b| a + b}
      end
      abort_codes
    end

    def build_results
      abort_codes = group_codes(data[:abort_count])
      total_abort_count = abort_codes.values.inject {|a, b| a+b}

      top_abort = extract_top_x(abort_codes, 5)

      top_abort.each do |abort|
        abort_name = resolve_abort_code(abort[:code])
        abort[:code] = abort_name
        if as_percent?
          abort_percentage = total_abort_count == 0 ? 0 : ((abort[:count] / total_abort_count.to_f) * 100).round
          abort[:count] = abort_percentage
        end
      end

      top_abort
    end

    def build_results_per_machine
      total_abort_count_hash = {}
      data[:abort_count].each do |machine_id, buckets|
        if timeframe_all?
          total_abort_count_hash.merge!(buckets) {|k, val1 , val2| val1 + val2}
        else
          buckets.each do |time, error_hash|
            total_abort_count_hash.merge!(error_hash) {|k, val1 , val2| val1 + val2}
          end
        end
      end
      top_5_aborts = extract_top_x(total_abort_count_hash, 5).map {|abort| abort[:code]}

      results = {}
      data[:abort_count].each do |machine_id, buckets|
        results[machine_id] ||= {}
        if timeframe_all?
          get_aborts_from_hash(buckets, results, top_5_aborts, machine_id)
        else
          buckets.each do |time, abort_hash|
            get_aborts_from_hash(abort_hash, results, top_5_aborts, machine_id)
          end
        end
      end
      results
    end

    def get_aborts_from_hash(abort_hash, results, top_5_aborts, machine_id, time = nil)
      top_5_aborts.map do |abort_code|
        abort_name = resolve_abort_code(abort_code)
        if time
          results[machine_id][time][abort_name] ||= 0
          # we want skipped aborts to have 0 count
          next if abort_hash[abort_code].nil?
          results[machine_id][time][abort_name] += abort_hash[abort_code]
        else
          results[machine_id][abort_name] ||= 0
          # we want skipped aborts to have 0 count
          next if abort_hash[abort_code].nil?
          results[machine_id][abort_name] += abort_hash[abort_code]
        end
      end
    end
  end
end