module Sideable
  extend ActiveSupport::Concern

  class InvalidSideType < StandardError; end
  class InvalidSideNum < StandardError; end

  module LensSide
    RIGHT = 'R'
    LEFT = 'L'
  end

  def side_letter
    return lens_side_num_to_L_R unless is_machine_data?

    # If the data is attached to a machine, the machine type dictates the lens side
    if machine.is_box_coater?
      lens_side_num_to_CC_CX
    else
      lens_side_num_to_L_R
    end
  end

  def side_num
    self[:side]
  end

  def side
    return lens_side_num_to_left_right unless is_machine_data?

    # If the data is attached to a machine, the machine type dictates the lens side
    if machine.is_box_coater?
      lens_side_num_to_concave_convex
    else
      lens_side_num_to_left_right
    end
  end

  def opposite_side_num
    return 0 if side_num == 1
    return 1 if side_num == 0

    raise InvalidSideNum
  end

  # This is invoked to ensure the machine type is still utilized when the side is not part of the model
  # Ex: Error model does not have a side property, use the one from its job (which is not associated to a machine)
  def self.side_letter_for_machine(side_num, machine)
    return lens_side_num_to_CC_CX(side_num) if machine.is_box_coater?
    lens_side_num_to_L_R(side_num)
  end

  def self.side_letter_to_side_num(side)
    side = side.downcase

    return 0 if %w[cc l].include? side
    return 1 if %w[cx r].include? side

    raise InvalidSideType
  end

  def self.side_to_side_num(side)
    side = side.to_sym
    return 0 if [:left, :concave].include?(side)
    1
  end

  def self.side_num_to_left_right(side_num)
    return :left if side_num == 0
    :right
  end

  private

  def is_machine_data?
    return true if respond_to? :machine
    false
  end

  def lens_side_num_to_left_right
    return :left if side_num == 0
    :right
  end

  def lens_side_num_to_concave_convex
    return :concave if side_num == 0
    :convex
  end

  def self.lens_side_num_to_L_R(side_num)
    return 'L' if side_num == 0
    'R'
  end

  def self.lens_side_num_to_CC_CX(side_num)
    return 'CC' if side_num == 0
    'CX'
  end

  def lens_side_num_to_L_R
    Sideable.lens_side_num_to_L_R(side_num)
  end

  def lens_side_num_to_CC_CX
    Sideable.lens_side_num_to_CC_CX(side_num)
  end
end
