# == Schema Information
#
# Table name: jobs
#
#  id              :bigint           not null, primary key
#  batch_criteria  :string(4000)
#  description     :string(4000)
#  enabled         :boolean
#  expiry_date     :datetime
#  job_type        :string(4000)
#  lmat_type       :string(4000)
#  manufacturer    :string(4000)
#  name            :string(4000)
#  number          :string(4000)
#  priority        :integer
#  sequence_number :integer
#  side            :integer
#  template        :string(4000)
#  created_at      :datetime         not null
#  updated_at      :datetime         not null
#  lmat_id         :string(4000)
#  order_id        :bigint
#
# Indexes
#
#  index_jobs_on_number                           (number)
#  index_jobs_on_number_and_side_and_expiry_date  (number,side,expiry_date)
#  index_jobs_on_order_id                         (order_id)
#
# Foreign Keys
#
#  fk_rails_...  (order_id => orders.id) ON DELETE => nullify
#
class Job < ApplicationRecord
  include Cloneable
  include Sideable
  include Pushable
  include Expirable
  include ExtractTagData

  has_many :problems, class_name: "Error"
  has_many :job_processes, -> { order("updated_at DESC") }
  has_many :job_process_parameters, through: :job_processes
  has_many :machines, through: :job_processes
  has_many :job_data
  has_many :job_manufacturing_processes
  has_many :manufacturing_processes, through: :job_manufacturing_processes
  has_many :job_capabilities
  has_many :capabilities, through: :job_capabilities
  has_many :job_states, -> { order("updated_at DESC") }
  has_one :block, required: false
  has_many :batch_job_data

  belongs_to :order, required: false, touch: true

  validates_presence_of :number

  class JobNumberAlreadyActive < StandardError; end

  def self.associations_to_clone
    [
      :job_data,
      job_capabilities: :capability,
      job_states: :machine,
      block: :block_reactivate_histories,
      job_manufacturing_processes: :manufacturing_process,
      job_processes: [
        :machine,
        :manufacturing_process,
        :job_process_parameters
      ],
      problems: :machine
    ]
  end

  def tags_from_job_data
    # Job data SHOULD only be for the job side, however, the Job API allows for data on the other side for some reason...
    tag_data = extract_tags_by_association(job_data)
    tag_data[side]
  end

  def last_job_state
    @last_job_state ||= job_states.first
  end

  def last_status_type
    if job_states.empty?
      nil
    else
      JobState.states[last_job_state&.state]
    end
  end

  def status
    return 'active' unless expired?

    active = FindHelpers::JobFinders.active_with_number_and_side_num(number, side_num)
    latest = FindHelpers::JobFinders.latest_expired_with_number_and_side_num(number, side_num)

    if active.nil? && latest.id == id
      "expired"
    else
      "archived"
    end
  end

  def revive
    active_order = Order.active.where(job_number: number).order("updated_at DESC").first
    active_job = FindHelpers::JobFinders.active_with_number_and_side_num(number, side_num)

    raise JobNumberAlreadyActive unless (active_job.nil? && active_order.nil?)

    reset_expiry

    # broadcast once revive is finished
    ActionCable.server.broadcast "everything", {
      model: JSON.parse(self.to_json),
      class: self.broadcast_class_name,
      action: :update,
    }
  end

  # Returns all job states, grouped by machine. Includes process params for finished states.
  def machine_history
    machine_history = {}
    job_states.each do |state|
      sn_key = state.machine.sn.to_sym
      machine_history[sn_key] = {
        name: state.machine.name,
        sn: state.machine.sn,
        description: state.machine.description,
        states: []
      } if machine_history[sn_key].nil?
      machine_history[sn_key][:states] << {state: state, process_params: state.process_params}
    end
    machine_history
  end

  # Returns a combination of the tags tied to the job, tags tied to the job's order and the job's attribute as tags
  def data_tags(include_processes = false)
    job_attributes = self.as_json(only: [:number, :description, :sequence_number, :priority, :manufacturer, :lmat_id, :lmat_type, :batch_criteria, :template, :job_type])
    job_attributes = job_attributes.map {|name, value| {name: name, value: value, type: 'base'}}
    job_tags = job_data.map {|datum| {name: datum[:name], value: datum[:value], type: 'tag'}}
    order_tags = order.nil? ? [] : order.order_data.where(side: side_num).map {|datum| {name: datum[:name], value: datum[:value], type: 'tag'}}
    return job_attributes + job_tags + order_tags unless include_processes
    mapped_processes = manufacturing_processes.map { |manufacturing_process| {name: manufacturing_process.name, type: 'process'} }
    job_attributes + job_tags + order_tags + mapped_processes
  end

  def self.combine_machine_history(left_history, right_history)
    combined_history = {}

    side_histories = [{
      side: :left,
      history: left_history
    },
    {
      side: :right,
      history: right_history
    }]

    side_histories.each do |side_history|

      side = side_history[:side]
      history = side_history[:history]

      next if history.nil?

      history.each do |key, value|
        machine = Machine.find_by_sn(value[:sn])
        combined_history[key] = {
          name: value[:name],
          sn: value[:sn],
          description: value[:description],
          states: {
            left: [],
            right: []
          },
          isBoxCoater: !machine ? false : machine.is_box_coater?
        } if combined_history[key].nil?
        combined_history[key][:states][side] = value[:states]
      end
    end
    combined_history
  end

  def opposite_job
    return nil if order.nil?
    order.jobs.where(side: opposite_side_num).first
  end

  def self.ensure_find_latest_job(number)
    latest_job, shard = [ActiveRecord::Base.connected_to(shard: :primary, role: :reading) { FindHelpers::JobFinders.latest_with_number(number) }, :primary]
    latest_job, shard = [ActiveRecord::Base.connected_to(shard: :archive, role: :reading) { FindHelpers::JobFinders.latest_with_number(number) }, :archive] if latest_job.nil?
    [latest_job, shard]
  end

  def self.ensure_job_uniqueness(number, side_num)
    job = FindHelpers::JobFinders.active_with_number_and_side_num(number, side_num)
    if job
      if job.order.nil?
        job.expire
      else
        job.order.expire_job
      end
    end
  end

  def self.delete_expired_before(expired_date)
    data_archive_settings = DataArchiveSetting.first
    max_delete_queue = data_archive_settings.delete_count_job
    return 0 if max_delete_queue.nil? || max_delete_queue <= 0
    deleted_objects = 0
    # Delete the loose jobs that have been expired before the given date
    # Only delete a maximum of `max_delete_queue` jobs
    # This only deletes the loose jobs (jobs that do not have an order_id)
    ActiveRecord::Base.connected_to(shard: :primary, role: :writing) do
      jobs = Job.where(order_id: nil).where(Job.arel_table[:expiry_date].lt(expired_date)).limit(max_delete_queue)
      jobs.each do |job| # Delete manufacturing processes associated to these jobs
        # This could be batched to optimize further (or avoid mssql 2000+ row deletion issue)
        deleted_objects += ManufacturingProcess.where(id: job.manufacturing_processes.pluck(:id)).delete_all
        deleted_objects += JobManufacturingProcess.where(id: job.job_manufacturing_processes.pluck(:id)).delete_all
      end
      deleted_objects += jobs.delete_all
    end
    deleted_objects
  end
end
