# == Schema Information
#
# Table name: machine_states
#
#  id                  :bigint           not null, primary key
#  addon               :string(4000)
#  calculated_duration :decimal(10, 3)
#  comment             :string(4000)
#  machine_sn          :string(4000)
#  state               :integer          default("on")
#  time                :datetime
#  created_at          :datetime         not null
#  updated_at          :datetime         not null
#
# Indexes
#
#  index_machine_states_on_machine_sn  (machine_sn)
#  index_machine_states_on_time        (time)
#
# Foreign Keys
#
#  fk_rails_...  (machine_sn => machines.sn) ON DELETE => cascade
#
class MachineState < ApplicationRecord
  include Pushable
  include ByMachineAndPeriod
  include ForwardMachineData
  include DataHelpers
  include BoxCoater::MachineStateHelper
  include ArchiveHelpers

  enum state: MACHINE_STATES
  belongs_to :machine, foreign_key: :machine_sn, primary_key: :sn, touch: true

  def date_str
    time.strftime("%Y/%m/%d")
  end

  def time_str
    time.strftime("%T.%3N")
  end

  def next_in_time
    # Check within both Archive and Primary since when you have the last machine state in Archive and you do next_in_time,
    # you want to find the first machine state in the Primary
    machine_state = nil
    # Since we dont want to unnecessarily query the archive, if machine_state.time is lesser than 32.days (archive time) dont check archive
    if time < data_expired_date
      ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
        machine_state = MachineState.where(machine_sn: machine_sn).where(MachineState.arel_table[:time].gt(time)).order(time: :asc).limit(1).first
      end
    end
    if machine_state.nil?
      ActiveRecord::Base.connected_to(shard: :primary, role: :reading) do
        machine_state = MachineState.where(machine_sn: machine_sn).where(MachineState.arel_table[:time].gt(time)).order(time: :asc).limit(1).first
      end
    end
    machine_state
  end

  def previous_in_time
    machine_state = nil
    ActiveRecord::Base.connected_to(shard: :primary, role: :reading) do
      machine_state = MachineState.where(machine_sn: machine_sn).where(MachineState.arel_table[:time].lt(time)).order(time: :desc).limit(1).first
    end
    if machine_state.nil?
      ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
        machine_state = MachineState.where(machine_sn: machine_sn).where(MachineState.arel_table[:time].lt(time)).order(time: :desc).limit(1).first
      end
    end
    machine_state
  end

  def forwarding_data
    {
      state: state,
      comment: comment,
      addon: addon
    }
  end

  def forwarding_path_sym
    :machine_state_path
  end

  def forwarding_params
    {
      sn: machine_sn,
      date: time.strftime("%Y-%m-%d"),
      time: time.strftime("%H:%M:%S,%L")
    }
  end

  def self.delete_before(date)
    data_archive_settings = DataArchiveSetting.first
    max_delete_queue = data_archive_settings.delete_count_machine_state
    return 0 if max_delete_queue.nil? || max_delete_queue <= 0
    deleted_objects = 0
    # Delete the machine states that have existed before the given date
    # Only delete a maximum of `max_delete_queue` machine states
    ActiveRecord::Base.connected_to(shard: :primary, role: :writing) do
      machine_states = MachineState.where(MachineState.arel_table[:time].lt(date)).limit(max_delete_queue)
      deleted_objects += machine_states.delete_all
    end
    deleted_objects
  end
end
