# == Schema Information
#
# Table name: orders
#
#  id             :bigint           not null, primary key
#  description    :string(4000)
#  expiry_date    :datetime
#  job_number     :string(4000)
#  name           :string(4000)
#  order_type     :string(4000)
#  patient        :string(4000)
#  priority       :integer
#  rtime          :time
#  workflow_state :string(4000)
#  wtime          :time
#  created_at     :datetime         not null
#  updated_at     :datetime         not null
#  address_id     :bigint
#  ship_to_id     :bigint
#
# Indexes
#
#  index_orders_on_address_id   (address_id)
#  index_orders_on_expiry_date  (expiry_date)
#  index_orders_on_job_number   (job_number)
#  index_orders_on_ship_to_id   (ship_to_id)
#  index_orders_on_updated_at   (updated_at)
#
# Foreign Keys
#
#  fk_rails_...  (address_id => addresses.id)
#  fk_rails_...  (ship_to_id => addresses.id)
#
class Order < ApplicationRecord
  include Cloneable
  include Pushable
  include Expirable
  include ExtractTagData

  belongs_to :ship_to, class_name: "Address", foreign_key: "ship_to_id", required: false, touch: true
  belongs_to :address, class_name: "Address", foreign_key: "address_id", required: false, touch: true
  has_many :order_data
  has_many :jobs
  has_many :job_manufacturing_processes, through: :jobs
  has_many :manufacturing_processes, through: :job_manufacturing_processes

  before_create do
    self.expiry_date = created_at + DataArchiveSetting.instance.job_max_age.hour
  end

  def self.associations_to_clone
    [
      :ship_to,
      :address,
      :order_data,
      jobs: Job.associations_to_clone
    ]
  end

  def revive
    active_order = Order.active.where(job_number: job_number).order("updated_at DESC").first
    active_job = Job.active.where(number: job_number).order("updated_at DESC").first
    if !active_order.nil? || !active_job.nil?
      raise "jobAlreadyActive"
    end
    # broadcast once revive is finished
    reset_expiry
    ActionCable.server.broadcast "everything", {
      model: JSON.parse(self.to_json),
      class: self.broadcast_class_name,
      action: :update,
    }
    jobs.each do |job|
      job.reset_expiry
      ActionCable.server.broadcast "everything", {
        model: JSON.parse(job.to_json),
        class: job.broadcast_class_name,
        action: :update,
      }
    end
  end

  def routing_launch_data
    tag_data = tags_from_order_data

    job_numbers = []

    jobs.each do |job|
      job_numbers << {jobNumber: job.number, lens: job.side}

      job_tag_data = job.tags_from_job_data


      tag_data[job.side][:lens] = job.side.to_s

      unless job_tag_data.nil?
        tag_data[job.side].merge!(job_tag_data)
      end

      job_processes = job.manufacturing_processes.pluck(:name)

      unless job_processes.length == 0
        tag_data[job.side][:processes] = job_processes.join(',')
      end
    end

    # If any of the side has no job, the side is not required
    tag_data[:left] = nil unless has_side?(:left)
    tag_data[:right] = nil unless has_side?(:right)

    {tag_data: tag_data, job_numbers: job_numbers}
  end

  def broadcast_update_order
    PushDataJob.perform_later('update_order', routing_launch_data)
  end

  def tags_from_order_data
    return extract_tags_by_association(order_data)
  end

  def self.ensure_find_by_id(id)
    order, shard = [ActiveRecord::Base.connected_to(shard: :primary, role: :reading) { Order.where(id: id).first }, :primary]
    order, shard = [ActiveRecord::Base.connected_to(shard: :archive, role: :reading) { Order.where(id: id).first }, :archive] if order.nil?
    [order, shard]
  end

  def wanted_date_str
    wtime&.strftime("%Y/%m/%d")
  end

  def wanted_time_str
    wtime&.strftime("%H:%M:%S.%L")
  end

  def requested_date_str
    rtime&.strftime("%Y/%m/%d")
  end

  def requested_time_str
    rtime&.strftime("%H:%M:%S.%L")
  end

  def expire_job
    self.expire
    jobs.each {|job| job.expire}
  end

  def self.ensure_order_uniqueness(number)
    order = FindHelpers::OrderFinders.find_active_by_number(number)
    if order
      order.expire_job
    end
  end

  def has_side?(side)
    jobs.select { |job| job.side == side}.size > 0
  end

  def self.delete_expired_before(expired_date)
    data_archive_settings = DataArchiveSetting.first
    max_delete_queue = data_archive_settings.delete_count_order
    return 0 if max_delete_queue.nil? || max_delete_queue <= 0
    deleted_objects = 0
    # Delete the orders that have been expired before the given date
    # Only delete a maximum of `max_delete_queue` orders
    #
    # Delete any job that is associated to those orders
    # An order can have up to 2 jobs associated to it,
    # meaning it deletes a maximum of (`max_delete_queue` * 2) jobs
    #
    # Delete any order_data that is associated to those orders
    ActiveRecord::Base.connected_to(shard: :primary, role: :writing) do
      orders = Order.where(Order.arel_table[:expiry_date].lt(expired_date)).limit(max_delete_queue)
      orders.each do |order|
        # This could be batched to optimize further (or avoid mssql 2000+ row deletion issue)
        deleted_objects += ManufacturingProcess.where(id: order.manufacturing_processes.pluck(:id)).delete_all
        deleted_objects += JobManufacturingProcess.where(id: order.job_manufacturing_processes.pluck(:id)).delete_all
        deleted_objects += Job.where(id: order.jobs.pluck(:id)).delete_all
        deleted_objects += OrderDatum.where(id: order.order_data.pluck(:id)).delete_all
      end
      deleted_objects += orders.delete_all
    end
    deleted_objects
  end
end
